import logging
import json
import os
from threading import Thread

import click

from .runner import Runner
from .crawler import Crawler
from . import severity
from .server import Server
from .settings import Settings


@click.group()
def cli():
    pass


@cli.command()
@click.argument("url")
@click.option("--severity", "-s")
@click.option("--fingerprint", "-f", multiple=True)
@click.option("--debug", is_flag=True)
@click.option("--crawl", is_flag=True)
@click.option("--output", type=click.Choice(["text", "json"]), default="text")
@click.option("--ignore-type", multiple=True)
@click.option("--setting", "-s", multiple=True)
def check(url, severity, debug, crawl, fingerprint, output, ignore_type, setting):

    for s in setting:
        k, v = s.split("=")
        try:
            value = json.loads(v)
        except json.JSONDecodeError:
            # assume string
            value = v
        key = k.lower()
        Settings[key] = value

    if os.path.exists(url) and os.path.isdir(url):
        click.secho(
            "Looks like you pointed to a directory, starting a server to check it...",
            fg="yellow",
        )
        server = Server(url)
        url = f"http://127.0.0.1:{server.port}"
        click.secho(f"Local site running at {url}", fg="yellow")
        thread = Thread(target=server.serve, daemon=True)
        thread.start()

    if debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    if crawl:
        click.echo(f"Crawling {url}...")
        crawler = Crawler()  # options
        crawler.crawl(url)
        urls = crawler.urls_found
    else:
        urls = [url]

    runner = Runner(
        urls, fingerprints=fingerprint, severity=severity, ignore_types=ignore_type
    )
    runner.run()

    if output == "text":

        failed = False

        for u, url_runner in runner.url_runners.items():
            if url_runner.issues:
                click.echo(u)
                failed = True
                click.secho(f"{len(url_runner.issues)} issues found", fg="red")
                click.echo()
                for issue_url, url_issues in url_runner.issues.by_url().items():
                    # TODO would be most useful to see these output by line
                    # and if a line contains multiple errors, make that look nice (some may be duplicate depending on check?)
                    click.echo("======================================")
                    click.echo(issue_url)
                    for issue in url_issues:
                        click.secho(issue.type, fg=color_for_severity(issue.severity))
                        click.echo(issue.fingerprint)
                        click.echo(issue.content)
                        if issue.extra:
                            click.echo(
                                json.dumps(issue.extra, indent=2, ensure_ascii=False)
                            )
                        click.echo()
                    click.echo()
        if failed:
            click.secho("\nⓧ Some checks failed!", fg="red")
            exit(1)
        else:
            click.secho("\n✓ All checks passed!", fg="green")

    elif output == "json":
        json_output = json.dumps(runner.as_data(), indent=2, sort_keys=True)
        print(json_output)

        for u, url_runner in runner.url_runners.items():
            if url_runner.issues:
                exit(1)


def color_for_severity(s):
    if s >= severity.ERROR:
        return "red"
    if s >= severity.WARNING:
        return "yellow"
    return None


if __name__ == "__main__":
    cli()
